/*
  Load applet - Maemo5 edition

*/

#include <gtk/gtk.h>
#include <hildon/hildon.h>
#include <glib/gerror.h>
#include <glib.h>
#include <glib/gstdio.h>
#include <string.h>
#include <osso-log.h>
#include <unistd.h>
#include <stdlib.h>
#include <signal.h>

#include "item.h"

#define LOAD_APPLET_STATUS_MENU_ITEM_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE (obj, LOAD_APPLET_TYPE_STATUS_MENU_ITEM, LoadAppletStatusMenuItemPrivate))

struct _LoadAppletStatusMenuItemPrivate {
	GtkWidget *button; 
	GtkWidget *buttonPicture;
	GtkWidget *buttonVideo;
	GtkWidget *buttonSettings;
	GtkWidget *icon;
	GdkPixbuf *pixbuf_bars;
	GdkPixbuf *pixbuf_full;
	GdkPixbuf *pixbuf_empty;
	GdkPixbuf *pixbuf_picture;
	GdkPixbuf *pixbuf_video;
	GdkPixbuf *pixbuf_settings;
	gulong screenshot_h;
	gulong video_h;
	pid_t rec_pid;
	gboolean recording;
	guint timeout_id;
	gint lastU, lastN, lastIO, lastI;
	guchar last_mem_level;
	guchar last_cpu_level;
};

HD_DEFINE_PLUGIN_MODULE (LoadAppletStatusMenuItem, load_applet_status_menu_item, HD_TYPE_STATUS_MENU_ITEM)



#define SHUTTER_SOUND "/usr/share/sounds/camera_snd_title_1.wav"
#define BEEP_SOUND    "/usr/share/sounds/ui-default_beep.wav" 
#define MAX_SSHOT_NAME_LEN 64
#define SSHOT_DEFAULT_NAME "screenshot"
static gchar *sshotfilename = NULL;
static char sshotname[MAX_SSHOT_NAME_LEN];
static guint screenshot_delay = 7;
static guint beeps = 0;
static gint sshotn = 0;




static void
load_applet_status_menu_item_class_finalize(LoadAppletStatusMenuItemClass * klass)
{
}

static void
load_applet_status_menu_item_dispose(GObject * object)
{
	G_OBJECT_CLASS(load_applet_status_menu_item_parent_class)->dispose(object);
}

static void
load_applet_status_menu_item_class_init(LoadAppletStatusMenuItemClass * klass)
{
	GObjectClass *object_class = G_OBJECT_CLASS(klass);

	object_class->dispose = load_applet_status_menu_item_dispose;

	g_type_class_add_private(klass, sizeof(LoadAppletStatusMenuItemPrivate));
}

/*
 * Get the filename for next screen-shot. Start with nr.
 * 
 */
static int
get_next_sshot(gint nr, const gchar *suffix)
{
	FILE *f;
	gint n;

	n = nr;
	f = NULL;
	do {
		g_snprintf (sshotname, MAX_SSHOT_NAME_LEN-1, "%s/MyDocs/.images/%s%02d.%s", getenv("HOME"), sshotfilename, n, suffix);
		f = g_fopen(sshotname, "r");
		if (f) {
			fclose(f); 
			n++;
		}
	}
	while(f != NULL);
		
	return n;
}



/*
 * Close the menu, make sure it is cosed and redrawn
 */
static void
do_little_dance (void)
{
	while (gtk_events_pending())
	{
		gtk_main_iteration();
	}

	usleep(1000);

	sched_yield();
}


/*
 *  Take screeshot using the root window drawable
 */
static gboolean
take_screenshot(void)
{
	int width, height;
	GdkDrawable *root_window;
	GdkPixbuf *pixbuf;
	gboolean ret;
	GError *error = NULL;
	
	do_little_dance();
	sshotn = get_next_sshot(sshotn, "png");


	root_window = gdk_get_default_root_window();
	gdk_drawable_get_size(root_window, &width, &height);
	pixbuf = gdk_pixbuf_get_from_drawable(NULL,
                                           root_window,
                                           gdk_drawable_get_colormap (root_window),
                                           0, 0,
                                           0, 0,
                                           width, height);
      

	ret = gdk_pixbuf_save(pixbuf, sshotname, "png", &error, NULL);
	g_object_unref(pixbuf);

	//DEBUG
	printf("\tSaving %s\n", sshotname);

	return FALSE;
}


/*
 * Either beep or play shutter sound and make the screenshot
 */
static gboolean
beep_or_shoot(gpointer data)
{
	beeps++;
	if (beeps >= screenshot_delay) {
		hildon_play_system_sound(SHUTTER_SOUND);
		take_screenshot();
		return FALSE; 
	} else {
		hildon_play_system_sound(BEEP_SOUND);
		return TRUE;  
	}
}

/*
 * Answer the callback by setting delayed screenshot
 */
static void
activate_delayed_screenshot_item(GtkMenuItem *item, gpointer data)
{
	beeps = 0;
	gtk_timeout_add(1000, beep_or_shoot, NULL);
}


/*
 * Answer the callback on video button
 * If idle, start recording
 * If already recording, stop
 */
static void
activate_video_item(GtkMenuItem *item, gpointer data)
{	
	LoadAppletStatusMenuItem *la_item = (LoadAppletStatusMenuItem*)data;
	pid_t rec_pid;
		
	if (la_item->priv->recording == FALSE) {
		// We are not recording. Start recording by exec	
		rec_pid = fork();
		if (rec_pid == 0) {
			execl("/usr/bin/gst-launch", "/usr/bin/gst-launch", "ximagesrc", "!", "video/x-raw-rgb,framerate=5/1", "!", "ffmpegcolorspace", "!", "jpegenc", "quality=30", "!", "avimux", "!", "filesink", "location=/home/user/MyDocs/.videos/screencast.avi", NULL); 
			exit(0);
		} else {
			la_item->priv->recording = TRUE;
			la_item->priv->rec_pid = rec_pid;
		}
	} else {
		// We are already recording	
		kill(la_item->priv->rec_pid, SIGTERM);
		g_debug("Killing %d", (int)la_item->priv->rec_pid);
		la_item->priv->recording = FALSE;
	}
}


/*
 * utility function to load icon into pixbuf 
 */
static GdkPixbuf*
la_load_pixmap(const gchar* location)
{
	GdkPixbuf *pixbuf;
	GError *error = NULL;

	pixbuf = gdk_pixbuf_new_from_file(location, &error); 
	if (error) {
		g_error("Loading %s failed with : %s", location, error->message);
		g_error_free (error);
		pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, 24, 24);
	}
	return pixbuf;
}



/*
 * Load icons into pixbufs
 */
static void
la_load_pixmaps(LoadAppletStatusMenuItemPrivate *priv)
{
	#define ICON_DIR   		"/usr/share/pixmaps/"
	#define ICON_EMPTY 		ICON_DIR"la_empty.png"
	#define ICON_FULL  		ICON_DIR"la_full.png"
	#define ICON_PICTURE  	ICON_DIR"la_picture.png"
	#define ICON_VIDEO  	ICON_DIR"la_video.png"
	#define ICON_SETTINGS  	ICON_DIR"la_settings.png"
	#define ICON_ICON  		ICON_DIR"la_icon.png"	
	
	priv->pixbuf_bars	  = gdk_pixbuf_new (GDK_COLORSPACE_RGB, TRUE, 8, 68, 40);
	gdk_pixbuf_fill(priv->pixbuf_bars, 0x00000000);
	priv->pixbuf_empty    = la_load_pixmap(ICON_EMPTY);
	priv->pixbuf_full     = la_load_pixmap(ICON_FULL);
	priv->pixbuf_picture  = la_load_pixmap(ICON_PICTURE);
	priv->pixbuf_video    = la_load_pixmap(ICON_VIDEO);
	priv->pixbuf_settings = la_load_pixmap(ICON_SETTINGS);
	
}



/*
 * Read current MEM usage and return indicator between 5 and 1 - how many bars are "full"
 */
static guchar
la_check_mem (LoadAppletStatusMenuItemPrivate *priv)
{
	#define MEMFILE "/proc/meminfo"
	#define MAX_READ_CHARS 128
	char read_buffer[MAX_READ_CHARS];
	FILE *fin;
	int mem_used = 0;
	int mem_total = 0;
	int mem_cached = 0;
	int mem_buffers = 0;
	int mem_free = 0;

	//Open the memory info file and get current free memory
	fin = fopen(MEMFILE, "r");
	if (fin == NULL) {
		g_warning("Can't open "MEMFILE"\n");
		return TRUE;
	}
	while (fgets(read_buffer, MAX_READ_CHARS, fin) != NULL) {
		if (strncmp(read_buffer, "MemTotal", 8) == 0) {
			sscanf(read_buffer + 10, "%d", &mem_total);
		} else if (strncmp(read_buffer, "MemFree", 6) == 0) {
			sscanf(read_buffer + 9, "%d", &mem_free);
		} else if (strncmp(read_buffer, "Buffers", 6) == 0) {
			sscanf(read_buffer + 9, "%d", &mem_buffers);
		} else if (strncmp(read_buffer, "Cached", 6) == 0) {
			sscanf(read_buffer + 8, "%d", &mem_cached);
			break;
		}
	}
	fclose(fin);
	mem_used = mem_total - mem_free - mem_buffers - mem_cached;

	//very scientific - whatever looks good
	if (mem_used > 0.9*mem_total)
		return 5;
	else if (mem_used > 0.7*mem_total)
		return 4;
	else if (mem_used > 0.5*mem_total)
		return 3;
	else if (mem_used > 0.4*mem_total)
		return 2;
	else
		return 1;
}

/*
 * Read current CPU usage and return indicator between 5 and 1 - how many bars are "full"
 */
static guchar
la_check_cpu (LoadAppletStatusMenuItemPrivate *priv)
{
	#define CPUFILE "/proc/stat"
	gint curU, curN, curIO, curI;
	gint deltaU, deltaN, deltaIO, deltaI;
	int load, idle;
	GError *error = NULL;
	gchar *contents;
	gsize lenght;
	gchar **splits;

	if (!g_file_get_contents (CPUFILE, &contents, &lenght, &error)) {
		fprintf (stderr, "ERR: can't read file %s: %s\n", CPUFILE, error->message);
		g_error_free (error);
		return 0;
	}
	
	splits = g_strsplit_set (contents, " ",  -1);

	sscanf(splits[2], "%d", &curU);
	sscanf(splits[3], "%d", &curN);
	sscanf(splits[4], "%d", &curIO);
	sscanf(splits[5], "%d", &curI);
	
	g_strfreev (splits);
	g_free (contents);
    
	idle = (curI - priv->lastI);
	if (idle == 0) load = 100;
	else load = 100-idle;
	if (load>100) load = 0;
	deltaU = curU - priv->lastU;
	deltaN = curN - priv->lastN;
	deltaIO = curIO - priv->lastIO;
	deltaI = curI - priv->lastI;
	priv->lastU = curU;
	priv->lastN = curN;
	priv->lastIO = curIO;
	priv->lastI = curI;

	if (load > 90)
		return 5;
	else if (load > 75)
		return 4;
	else if (load > 50)
		return 3;
	else if (load > 25)
		return 2;
	else
		return 1;
}


/*
 * Compose and blit the current status of memory bars
 */
static void
la_blit_memory_bars (const guchar level, LoadAppletStatusMenuItemPrivate *priv)
{
	guint x, y;

	x = 0;
	y = 0;
	if (level > 0)
		gdk_pixbuf_composite(priv->pixbuf_full, priv->pixbuf_bars, x, y, 12, 19, x, y, 1, 1, GDK_INTERP_NEAREST, 255);
	else
		gdk_pixbuf_composite(priv->pixbuf_empty, priv->pixbuf_bars, x, y, 12, 19, x, y, 1, 1, GDK_INTERP_NEAREST, 255);
	x = 14;
	if (level > 1)
		gdk_pixbuf_composite(priv->pixbuf_full, priv->pixbuf_bars, x, y, 12, 19, x, y, 1, 1, GDK_INTERP_NEAREST, 255);
	else
		gdk_pixbuf_composite(priv->pixbuf_empty, priv->pixbuf_bars, x, y, 12, 19, x, y, 1, 1, GDK_INTERP_NEAREST, 255);
	x = 28;
	if (level > 2)
		gdk_pixbuf_composite(priv->pixbuf_full, priv->pixbuf_bars, x, y, 12, 19, x, y, 1, 1, GDK_INTERP_NEAREST, 255);
	else
		gdk_pixbuf_composite(priv->pixbuf_empty, priv->pixbuf_bars, x, y, 12, 19, x, y, 1, 1, GDK_INTERP_NEAREST, 255);
	x = 42;
	if (level > 3)
		gdk_pixbuf_composite(priv->pixbuf_full, priv->pixbuf_bars, x, y, 12, 19, x, y, 1, 1, GDK_INTERP_NEAREST, 255);
	else
		gdk_pixbuf_composite(priv->pixbuf_empty, priv->pixbuf_bars, x, y, 12, 19, x, y, 1, 1, GDK_INTERP_NEAREST, 255);
	x = 56;
	if (level > 4)
		gdk_pixbuf_composite(priv->pixbuf_full, priv->pixbuf_bars, x, y, 12, 19, x, y, 1, 1, GDK_INTERP_NEAREST, 255);
	else
		gdk_pixbuf_composite(priv->pixbuf_empty, priv->pixbuf_bars, x, y, 12, 19, x, y, 1, 1, GDK_INTERP_NEAREST, 255);
}


/* 
 * Compose and blit current status of CPU bars
 */
static void
la_blit_cpu_bars (const guchar level, LoadAppletStatusMenuItemPrivate *priv)
{
	guint x, y;

	x = 0;
	y = 21;
	if (level > 0)
		gdk_pixbuf_composite(priv->pixbuf_full,  priv->pixbuf_bars, x, y, 12, 19, x, y, 1, 1, GDK_INTERP_NEAREST, 255);
	else
		gdk_pixbuf_composite(priv->pixbuf_empty, priv->pixbuf_bars, x, y, 12, 19, x, y, 1, 1, GDK_INTERP_NEAREST, 255);
	x = 14;
	if (level > 1)
		gdk_pixbuf_composite(priv->pixbuf_full,  priv->pixbuf_bars, x, y, 12, 19, x, y, 1, 1, GDK_INTERP_NEAREST, 255);
	else
		gdk_pixbuf_composite(priv->pixbuf_empty, priv->pixbuf_bars, x, y, 12, 19, x, y, 1, 1, GDK_INTERP_NEAREST, 255);
	x = 28;
	if (level > 2)
		gdk_pixbuf_composite(priv->pixbuf_full,  priv->pixbuf_bars, x, y, 12, 19, x, y, 1, 1, GDK_INTERP_NEAREST, 255);
	else
		gdk_pixbuf_composite(priv->pixbuf_empty, priv->pixbuf_bars, x, y, 12, 19, x, y, 1, 1, GDK_INTERP_NEAREST, 255);
	x = 42;
	if (level > 3)
		gdk_pixbuf_composite(priv->pixbuf_full,  priv->pixbuf_bars, x, y, 12, 19, x, y, 1, 1, GDK_INTERP_NEAREST, 255);
	else
		gdk_pixbuf_composite(priv->pixbuf_empty, priv->pixbuf_bars, x, y, 12, 19, x, y, 1, 1, GDK_INTERP_NEAREST, 255);
	x = 56;
	if (level > 4)
		gdk_pixbuf_composite(priv->pixbuf_full,  priv->pixbuf_bars, x, y, 12, 19, x, y, 1, 1, GDK_INTERP_NEAREST, 255);
	else
		gdk_pixbuf_composite(priv->pixbuf_empty, priv->pixbuf_bars, x, y, 12, 19, x, y, 1, 1, GDK_INTERP_NEAREST, 255);
}


/*
 * Ran to check and update the CPU and memory reading
 */
static gboolean
la_check_load (gpointer data)
{
	guchar current_cpu_level;
	guchar current_mem_level;
	LoadAppletStatusMenuItemPrivate *priv = (LoadAppletStatusMenuItemPrivate*)data;
   
	current_cpu_level = la_check_cpu(priv); 
	current_mem_level = la_check_mem(priv);

	//Update and blit only if data changed!
	if ((current_mem_level != priv->last_mem_level) || (current_cpu_level != priv->last_cpu_level)) {
		//gdk_pixbuf_fill(priv->pixbuf_current, 0x00000000); //Why?
		la_blit_memory_bars (current_mem_level, priv);
		la_blit_cpu_bars (current_cpu_level, priv);
		gtk_image_set_from_pixbuf(GTK_IMAGE(priv->icon), priv->pixbuf_bars);
		priv->last_mem_level = current_mem_level;
		priv->last_cpu_level = current_cpu_level;
	}

	return TRUE;  
}



static void
load_applet_status_menu_item_init (LoadAppletStatusMenuItem * menu_item)
{
	GtkWidget *hbox, *icon;

	menu_item->priv = LOAD_APPLET_STATUS_MENU_ITEM_GET_PRIVATE (menu_item);

	hbox = gtk_hbox_new(FALSE, 0);

	//Load stuff
	sshotfilename = g_strdup(SSHOT_DEFAULT_NAME);
	la_load_pixmaps(menu_item->priv);
	
/*
	menu_item->priv->button = gtk_button_new();
	gtk_widget_show(menu_item->priv->button);
	gtk_box_pack_start(GTK_BOX(hbox), menu_item->priv->button, TRUE, TRUE, HILDON_MARGIN_DEFAULT);
	menu_item->priv->screenshot_h = g_signal_connect(menu_item->priv->button, "clicked", G_CALLBACK(activate_delayed_screenshot_item), menu_item);
	menu_item->priv->icon = gtk_image_new_from_pixbuf(NULL);
	gtk_widget_show(menu_item->priv->icon);
	gtk_container_add(GTK_CONTAINER(menu_item->priv->button), GTK_WIDGET(menu_item->priv->icon));
	 */

	menu_item->priv->icon = gtk_image_new_from_pixbuf(NULL);
	gtk_widget_show(menu_item->priv->icon);
	gtk_box_pack_start(GTK_BOX(hbox), menu_item->priv->icon, TRUE, TRUE, 1);
	// Button Video
	menu_item->priv->buttonVideo = gtk_toggle_button_new();
	gtk_widget_show(menu_item->priv->buttonVideo);
	icon = gtk_image_new_from_pixbuf(menu_item->priv->pixbuf_video);
	gtk_widget_show(icon);
	gtk_button_set_image(GTK_BUTTON(menu_item->priv->buttonVideo), GTK_WIDGET(icon));
	gtk_box_pack_start(GTK_BOX(hbox), menu_item->priv->buttonVideo, TRUE, TRUE, 1);
	menu_item->priv->screenshot_h = g_signal_connect(menu_item->priv->buttonVideo, "clicked", G_CALLBACK(activate_video_item), menu_item);
	// Button Picture
	menu_item->priv->buttonPicture = gtk_button_new();
	gtk_widget_show(menu_item->priv->buttonPicture);
	icon = gtk_image_new_from_pixbuf(menu_item->priv->pixbuf_picture);
	gtk_widget_show(icon);
	gtk_button_set_image(GTK_BUTTON(menu_item->priv->buttonPicture), GTK_WIDGET(icon));
	gtk_box_pack_start(GTK_BOX(hbox), menu_item->priv->buttonPicture, TRUE, TRUE, 1);  
	menu_item->priv->screenshot_h = g_signal_connect(menu_item->priv->buttonPicture, "clicked", G_CALLBACK(activate_delayed_screenshot_item), menu_item);
	// Button Settings
/*
	menu_item->priv->buttonSettings = gtk_button_new();
	gtk_widget_show(menu_item->priv->buttonSettings);
	icon = gtk_image_new_from_pixbuf(menu_item->priv->pixbuf_settings);
	gtk_widget_show(icon);
	gtk_button_set_image(GTK_BUTTON(menu_item->priv->buttonSettings), GTK_WIDGET(icon));
	gtk_box_pack_start(GTK_BOX(hbox), menu_item->priv->buttonSettings, TRUE, TRUE, 1); 
*/
	// Pack widgets to menu
	gtk_widget_show(hbox);
	gtk_container_add(GTK_CONTAINER(menu_item), hbox);
	gtk_widget_show(GTK_WIDGET(menu_item));
	
	// More init

	menu_item->priv->last_mem_level = 0;
	menu_item->priv->last_cpu_level = 0;
	menu_item->priv->recording = FALSE;

	//TODO - do not run this in loop all time! Run in loop only when menu is open!!!!
	menu_item->priv->timeout_id = gtk_timeout_add(1000, la_check_load, menu_item->priv);
}



